@@ -19,7 +19,6 @@ from ruamel.yaml import YAML
 from tqdm import tqdm
 
 from .typeoverloads import FilePath
-from .widgets import ScrollableErrorDialog
 from .params.prjparam import PrjParam
 
 yaml = YAML()
@@ -136,6 +135,7 @@ def makeExceptionsShowDialogs(win: QtWidgets.QMainWindow,
   prevent that from happening.
   """
   app = QtWidgets.QApplication.instance()
+  from utilitys.widgets import ScrollableErrorDialog
 
   # Procedure taken from https://stackoverflow.com/a/40674244/9463643
   def new_except_hook(etype, evalue, tb):
@@ -175,6 +175,12 @@ def raiseErrorLater(err: Exception):
     raise err
   QtCore.QTimer.singleShot(0, _raise)
 
+def warnLater(msg: str, type_=Warning):
+  if not usingPostponedErrors:
+    warn(msg, type_)
+  else:
+    QtCore.QTimer.singleShot(0, lambda: warn(msg, type_))
+
 def pascalCaseToTitle(name: str, addSpaces=True) -> str:
   """
   Helper utility to turn a PascalCase name to a 'Title Case' title
@@ -325,14 +331,44 @@ def params_flattened(param: Parameter):
   return addList
 
 
-def flexibleParamTree(topParam:Parameter=None, showTop=True):
+def flexibleParamTree(topParam:Parameter, showTop=True):
   tree = ParameterTree()
   tree.setTextElideMode(QtCore.Qt.ElideRight)
   tree.header().setSectionResizeMode(QtWidgets.QHeaderView.Interactive)
-  if topParam is not None:
-    tree.setParameters(topParam, showTop)
+  tree.setParameters(topParam, showTop)
+  def hookupSignals(p: Parameter):
+    for ch in p:
+      hookupSignals(ch)
+    p.sigChildAdded.connect(lambda _p, _ch: hookupSignals(_ch))
+    setParamTooltips(tree)
+  hookupSignals(topParam)
   return tree
 
+def setParamsExpanded(tree: ParameterTree, expandedVal=True):
+  for item in tree.topLevelItems():
+    for ii in range(item.childCount()):
+      item.child(ii).setExpanded(expandedVal)
+  tree.resizeColumnToContents(0)
+
+
+def setParamTooltips(tree: ParameterTree, expandNameCol=False):
+  iterator = QtWidgets.QTreeWidgetItemIterator(tree)
+  item: QtWidgets.QTreeWidgetItem = iterator.value()
+  while item is not None:
+    # TODO: Set word wrap on long labels. Currently either can show '...' or wrap but not
+    #   both
+    # if tree.itemWidget(item, 0) is None:
+    #   lbl = QtWidgets.QLabel(item.text(0))
+    #   tree.setItemWidget(item, 0, lbl)
+    if (hasattr(item, 'param')
+        and 'tip' in item.param.opts
+        and len(item.toolTip(0)) == 0
+        and tree.itemWidget(item, 0) is None):
+      item.setToolTip(0, item.param.opts['tip'])
+    iterator += 1
+    item = iterator.value()
+  if expandNameCol:
+    setParamsExpanded(tree, True)
 
 def resolveYamlDict(cfgFname: FilePath, cfgDict: dict=None):
   if cfgDict is not None:
@@ -484,18 +520,3 @@ def popupFilePicker(parent=None, winTitle: str='', fileFilter: str='', existing=
     return fList[0]
   else:
     return None
-
-
-def helpTextToRichText(helpText: str, prependText='', postfixText=''):
-  # Outside <qt> tags
-  if helpText.startswith('<qt>'):
-    unwrappedHelpText = helpText[4:-5]
-  else:
-    unwrappedHelpText = helpText
-  if len(prependText) > 0 and len(helpText) > 0 or len(postfixText) > 0:
-    prependText += '<br>'
-  curText = prependText + unwrappedHelpText
-  if len(postfixText) > 0:
-    curText += '<br>' + postfixText
-  newHelpText = f'<qt>{curText}</qt>'
-  return newHelpText
\ No newline at end of file
