@@ -15,11 +15,10 @@ from .. import fns
 from ..processing import *
 from ..typeoverloads import FilePath
 import utilitys
-from ..fns import params_flattened, warnLater
+from ..fns import warnLater
 
 __all__ = [
-  'RunOpts', 'ParamEditor', 'ParamEditorDockGrouping', 'EditorPropsMixin', 'SPAWNED_EDITORS',
-  'params_flattened'
+  'RunOpts', 'ParamEditor', 'ParamEditorDockGrouping', 'EditorPropsMixin', 'SPAWNED_EDITORS'
 ]
 
 Signal = QtCore.Signal
@@ -410,9 +409,7 @@ class ParamEditor(QtWidgets.QDockWidget):
       name = groupName
     if not name.endswith('Tools'):
       name = name + ' Tools'
-    toolsEditor = cls(saveDir=None,
-                              fileType=lowerGroupName.replace(' ', ''),
-                              name=name, useNewInit=False)
+    toolsEditor = cls(fileType=lowerGroupName.replace(' ', ''), name=name)
     for btn in (toolsEditor.saveAsBtn, toolsEditor.applyBtn, toolsEditor.expandAllBtn,
                 toolsEditor.collapseAllBtn):
       btn.hide()
