@@ -2,23 +2,29 @@ from __future__ import annotations
 
 import typing as t
 from textwrap import wrap
+from warnings import warn
 
+import cv2 as cv
 import numpy as np
 import pandas as pd
 import pyqtgraph as pg
-import cv2 as cv
+from matplotlib import cm
+from matplotlib.pyplot import colormaps
 from pyqtgraph.Qt import QtWidgets, QtGui, QtCore
 # Taken directly from https://stackoverflow.com/a/20610786/9463643
+from pyqtgraph.graphicsItems.LegendItem import ItemSample
+from pyqtgraph.graphicsItems.ScatterPlotItem import drawSymbol
 from pyqtgraph.graphicsItems.ViewBox.ViewBoxMenu import ViewBoxMenu
 from pyqtgraph.parametertree import Parameter
 from typing_extensions import Literal
-
+from utilitys.fns import nameFormatter, pascalCaseToTitle, params_flattened, \
+  dynamicDocstring, getParamChild
 from utilitys.misc import CompositionMixin
 from utilitys.typeoverloads import FilePath
+
 from . import params
 from .constants import PrjEnums
 from .params.prjparam import PrjParam
-from .params import parameditor
 
 try:
   from pyqtgraph.Qt import QtWidgets
@@ -327,7 +333,7 @@ class ButtonCollection(QtWidgets.QGroupBox):
     Adds a button to a group based on the parameter. Also works for group params
     that have an acttion nested.
     """
-    for param in params.parameditor.params_flattened(param):
+    for param in params_flattened(param):
       curCopy = copy
       if param.type() in ['action', 'shortcut'] and param.opts.get('guibtn', True):
         existingBtn = None
@@ -452,15 +458,12 @@ class HasEasyChild(QtWidgets.QMainWindow):
   easyChild: EasyWidget
 
 
-class ImgViewer(CompositionMixin, parameditor.EditorPropsMixin, pg.PlotWidget):
+class ImageViewer(CompositionMixin, pg.PlotWidget):
 
 
-  @classmethod
-  def __initEditorParams__(cls):
-    cls.toolsEditor = parameditor.ParamEditor.buildClsToolsEditor(cls, 'Region Tools')
-
   def __init__(self, imgSrc: np.ndarray=None, **kwargs):
     super().__init__(**kwargs)
+    self.toolsEditor = params.ParamEditor(name='Region Tools')
     vb = self.getViewBox()
     self.menu: QtWidgets.QMenu = vb.menu
     self.menu.clear()
@@ -523,115 +526,233 @@ class ImgViewer(CompositionMixin, parameditor.EditorPropsMixin, pg.PlotWidget):
     else:
       self.imgItem.setImage(imgSrc)
 
+class CompositorItemSample(ItemSample):
+  def paint(self, p: QtGui.QPainter, *args):
+    opts = self.item.opts
 
-# class ImgCompositor(ImgViewer):
-#   def __init__(self, img: np.ndarray=None):
-#     super().__init__(img)
-#     # -----
-#     # Create properties
-#     # -----
-#     self.winSz = self.size().width(), self.size().height()
-#     self.baseImg = self.mainImg.imgItem
-#     self.viewbox = self.mainImg.getViewBox()
-#     self.legend = pg.LegendItem(offset=(5,5), brush=[0,0,0])
-#     self.addedItems: List[pg.ImageItem] = []
-#
-#     self.legendFontArgs = {'size': '13pt', 'color': 'w', 'bold': True, 'italic': False}
-#
-#     # -----
-#     # Configure relationships
-#     # -----
-#     self.legend.setParentItem(self.mainImg.plotItem)
-#     for ax in 'left', 'bottom', 'right', 'top':
-#       self.mainImg.plotItem.hideAxis(ax)
-#     self.mainImg.setContentsMargins(0, 0, 0, 0)
-#     self.baseImg.setZValue(1)
-#
-#   def setImage(self, baseImg: NChanImg, clearOverlays=True):
-#     self.winSz = baseImg.shape[:2][::-1]
-#     self.viewbox.setRange(
-#       xRange=[0, baseImg.shape[1]], yRange=[0, baseImg.shape[0]], padding=0
-#     )
-#     self.baseImg.setImage(baseImg)
-#     self.refreshWinContents()
-#     if clearOverlays:
-#       self.clearOverlays()
-#
-#   def addMasks(self, masks: List[BlackWhiteImg], names: np.ndarray=None, alpha=0.5,
-#                colors: list=None, removeBlank=True):
-#     backgroundClr = [0,0,0,0]
-#     if colors is None:
-#       colors = cm.get_cmap('viridis', len(masks)).colors*255
-#     if names is None:
-#       self.legend.hide()
-#       names = [None]*len(masks)
-#     else:
-#       self.legend.show()
-#
-#     if removeBlank:
-#       # Take out masks that are totally blank. Iterate in reverse order to avoid
-#       # index modification of untouched elements during deletion
-#       # Waiting until here to remove blank masks ensures the colors
-#       # will stay the same for all names if several images are overlayed
-#       idxsToKeep = list(range(len(masks)))
-#       for ii in reversed(idxsToKeep):
-#         if not np.any(masks[ii]):
-#           for lst in masks, names, idxsToKeep:
-#             del lst[ii]
-#       # 'Colors' is a np array, so we can't delete from it in the same loop
-#       colors = colors[idxsToKeep,:]
-#
-#
-#     curZ = 2
-#     lastLegIdx = len(self.legend.items)
-#     for name, color in zip(names, colors):
-#       brush, pen = pg.mkBrush(color), pg.mkPen(color)
-#       curScat = pg.ScatterPlotItem(symbol='s', brush=brush, pen=pen, width=5)
-#       self.legend.addItem(curScat, name=name)
-#     self.setLegendFontStyle(lastLegIdx, **self.legendFontArgs)
-#     for mask, color in zip(masks, colors):
-#       curLut = np.array([backgroundClr, color])
-#       toAdd = pg.ImageItem(mask, levels=[0,1], lut=curLut)
-#       toAdd.setZValue(curZ)
-#       toAdd.setOpacity(alpha)
-#       self.addedItems.append(toAdd)
-#       self.viewbox.addItem(toAdd)
-#       curZ += 1
-#     self.refreshWinContents()
-#
-#   def setLegendFontStyle(self, startItemIdx=0, **lblTxtArgs):
-#     for item in self.legend.items[startItemIdx:]:
-#       for single_item in item:
-#         if isinstance(single_item, pg.LabelItem):
-#           single_item.setText(single_item.text, **lblTxtArgs)
-#
-#   def clearOverlays(self):
-#     for imgItem in self.addedItems:
-#       self.viewbox.removeItem(imgItem)
-#     self.addedItems = []
-#     self.legend.clear()
-#
-#   def toNumpy(self) -> RgbaImg:
-#     exporter = ImageExporter(self.viewbox)
-#     outQArr = exporter.export(toBytes=True)
-#     return np.dstack((rgb_view(outQArr), alpha_view(outQArr)))
-#
-#   def save(self, saveFile: Union[str, Path]) -> bool:
-#     exporter = ImageExporter(self.viewbox)
-#     success = exporter.export(saveFile)
-#     return success
-#
-#   def refreshWinContents(self):
-#     # For some reason, processing events causes window size to screw up on export.
-#     # To fix this, every time win events are processed force another resize.
-#     # Find workaround?
-#     pg.mkQApp().processEvents()
-#     self.resize(*self.winSz)
-#     self.viewbox.resize(*self.winSz)
-#
-#   def show_exec(self):
-#     super().show()
-#     pg.mkQApp().exec_()
+    if opts.get('antialias'):
+      p.setRenderHint(p.Antialiasing)
+    symbol = opts.get('symbol', None)
+    p.translate(0, 14)
+    drawSymbol(p, symbol, opts['size'], pg.fn.mkPen(opts['pen']),
+               pg.fn.mkBrush(opts['brush']))
+
+class CompositorLegend(pg.LegendItem):
+
+  def paint(self, p: QtGui.QPainter, *args):
+    br = self.boundingRect()
+    p.setPen(self.opts['pen'])
+    p.setBrush(self.opts['brush'])
+    p.drawRoundedRect(br, 5, 5)
+
+class MaskCompositor(ImageViewer):
+  def __init__(self, img: np.ndarray=None):
+    super().__init__()
+    # -----
+    # Create properties
+    # -----
+    self.legend = CompositorLegend(offset=(5,5), horSpacing=5, brush='cccd', pen='k')
+    self.recordDf = pd.DataFrame(columns=['name', 'item', 'scatter', 'opacity']).set_index('name')
+
+    self.legendFontArgs = {'size': '11pt', 'color': 'k', 'bold': True}
+    self.nextZ = 2
+
+    # -----
+    # Configure relationships
+    # -----
+    self.legend.setParentItem(self.plotItem)
+    self.viewbox: pg.ViewBox = self.getViewBox()
+
+    self.masksParam = getParamChild(self.toolsEditor.params, 'Masks')
+    self.allVisible = True
+    # for ax in 'left', 'bottom', 'right', 'top':
+    #   self.mainImg.plotItem.hideAxis(ax)
+    # self.mainImg.setContentsMargins(0, 0, 0, 0)
+    def newFmt(name):
+      if name.startswith('set'):
+        name = name.replace('set', '')
+      name = pascalCaseToTitle(name)
+      return name
+    with nameFormatter.set(newFmt):
+      self.toolsEditor.registerFunc(self.setOverlayProperties, runOpts=params.RunOpts.ON_CHANGED)
+      self.toolsEditor.registerFunc(self.save)
+      self.toolsEditor.registerFunc(lambda: self.imgItem.setVisible(not self.imgItem.isVisible()),
+                                    name='Toggle Image Visible')
+      self.toolsEditor.registerFunc(lambda: self.legend.setVisible(not self.legend.isVisible()),
+                                    name='Toggle Legend Visible')
+      self.toolsEditor.registerFunc(self.toggleAllVisible, namePath=('Masks',))
+    if img is not None:
+      self.setImage(img)
+
+  def _addItemCtrls(self, record: pd.Series):
+    item = record['item']
+    def maskOpts(_, visible=True):
+      item.setVisible(visible)
+    newParam = getParamChild(self.masksParam,
+                             chOpts={'name': record.name, 'type': 'bool', 'value': True})
+    newParam.sigValueChanged.connect(maskOpts)
+
+  def setBaseImage(self, baseImg: np.ndarray, clearOverlays=True):
+    # self.winSz = baseImg.shape[:2][::-1]
+    # self.viewbox.setRange(
+    #   xRange=[0, imgItem.shape[1]], yRange=[0, imgItem.shape[0]], padding=0
+    # )
+    self.setImage(baseImg)
+    # self.refreshWinContents()
+    if clearOverlays:
+      self.clearOverlays()
+
+
+  def addMask(self, mask: np.ndarray, name:str=None, ignoreIfBlank=False, update=True):
+    if ignoreIfBlank and not np.any(mask):
+      return
+    newRecord = {}
+
+    if name is None:
+      name = '[No Name]'
+    name = self._getUniqueName(name)
+
+    curScat = pg.ScatterPlotItem(symbol='s', width=5)
+    self.legend.addItem(CompositorItemSample(curScat), name=name)
+    newRecord['scatter'] = curScat
+
+    toAdd = pg.ImageItem(mask)
+    toAdd.setZValue(self.nextZ+1)
+    self.viewbox.addItem(toAdd)
+    newRecord['item'] = toAdd
+
+    newRecord['opacity'] = -1
+
+    rec = pd.Series(newRecord, name=name)
+    self.recordDf = self.recordDf.append(rec)
+    self.nextZ += 1
+
+    self._addItemCtrls(rec)
+
+    if update:
+      self._updateGraphics()
+
+  def addMasks(self, masks: t.List[np.ndarray], names: t.Sequence[t.Optional[str]]=None, ignoreIfBlank=False):
+    if names is None:
+      names = [None]*len(masks)
+    for mask, name in zip(masks, names):
+      self.addMask(mask, name, ignoreIfBlank, update=False)
+    self._updateGraphics()
+
+  def _getUniqueName(self, baseName:str):
+    ii = 2
+    name = baseName
+    while name in self.recordDf.index:
+      name = f'{baseName} {ii}'
+      ii += 1
+    return name
+
+  def addLabelImg(self, labelImg: np.ndarray, name='Label', ignoreBackground=True):
+    """
+    Splits a label mask into its constituent labels and adds a mask for each unique label.
+
+    :param labelImg: Grayscale label mask of integers
+    :param name: Base name of each label. E.g. if 3 labels are present in the image, the
+      resulting masks will be named <name> 1, <name> 2, <name 3>
+    :param ignoreBackground: Whether a mask should be added for 0-valued labels (typically
+      0 is used to denote background / non-label values)
+    """
+    labels = np.unique(labelImg)
+    if ignoreBackground:
+      labels = labels[labels > 0]
+    newMasks = [labelImg == label for label in labels]
+    newNames = [name]*len(newMasks)
+    self.addMasks(newMasks, newNames)
+
+
+  def setLegendFontStyle(self, startItemIdx=0, **lblTxtArgs):
+    for item in self.legend.items[startItemIdx:]:
+      for single_item in item:
+        if isinstance(single_item, pg.LabelItem):
+          single_item.setText(single_item.text, **lblTxtArgs)
+
+  @dynamicDocstring(cmapVals=colormaps())
+  def setOverlayProperties(self, colormap='Set1', opacity=0.5):
+    """
+    Sets overlay properties
+    :param colormap:
+      pType: popuplineeditor
+      limits: {cmapVals}
+    :param opacity:
+      limits: [0,1]
+      step: 0.1
+    """
+    names = self.recordDf.index
+    numMasks = len(names)
+    cmap = cm.get_cmap(colormap, numMasks)
+    colors = cmap(np.arange(numMasks))
+    opacity = np.where(self.recordDf['opacity'] < 0, opacity, self.recordDf['opacity'])
+    colors[:,-1] = opacity
+    colors = (colors*255).astype('uint8')
+    backgroundClr = [0,0,0,0]
+    for name, color in zip(names, colors):
+      # Make sure alpha is not carried over
+      brush, pen = pg.mkBrush(color[:-1]), pg.mkPen(color[:-1])
+      self.recordDf.at[name, 'scatter'].setData(symbol='s', brush=brush, pen=pen, width=5)
+    for item, color in zip(self.recordDf['item'], colors):
+      curLut = np.array([backgroundClr, color])
+      item.setOpts(levels=[0,1], lut=curLut)
+    self.setLegendFontStyle(**self.legendFontArgs)
+
+  def _updateGraphics(self):
+    self.setOverlayProperties()
+
+  def clearOverlays(self):
+    for imgItem in self.recordDf['item']:
+      self.viewbox.removeItem(imgItem)
+    self.recordDf = self.recordDf.drop(self.recordDf.index)
+    self.legend.clear()
+    self.toolsEditor.params.child('Masks').clear()
+
+  def toggleAllVisible(self):
+    for param in self.masksParam:
+      if param.type() == 'bool':
+        param.setValue(not self.allVisible)
+    self.allVisible = not self.allVisible
+
+  def save(self, saveFile: FilePath='', cropToViewbox=False, toClipboard=False) -> bool:
+    """
+    :param saveFile:
+      helpText: Save destination. If blank, no file is created.
+      existing: False
+      pType: filepicker
+    :param toClipboard: Whether to copy to clipboard
+    :param cropToViewbox: Whether to only save the visible portion of the image
+    """
+    pm = self.imgItem.getPixmap()
+    painter = QtGui.QPainter(pm)
+    # TODO: Get legend exporting too
+    visibleMasks = [p.name() for p in params_flattened(self.masksParam) if p.value()]
+    for ii, (name, entry) in enumerate(self.recordDf.iterrows()):
+      if name in visibleMasks:
+        entry['item'].paint(painter)
+        legItems = self.legend.items[ii]
+        # legItems[0].paint(painter)
+        # legItems[1].paint(painter, None)
+    painter.end()
+    saveImg = pm.toImage()
+    if cropToViewbox:
+      maxBnd = np.array(self.image.shape[:2][::-1]).reshape(-1,1) - 1
+      vRange = np.array(self.viewbox.viewRange()).astype(int)
+      vRange = np.clip(vRange, 0, maxBnd)
+      # Convert range to topleft->bottomright
+      pts = QtCore.QPoint(*vRange[:,0]), QtCore.QPoint(*vRange[:,1])
+      saveImg = saveImg.copy(QtCore.QRect(*pts))
+    if toClipboard:
+      QtWidgets.QApplication.clipboard().setImage(saveImg)
+    saveFile = str(saveFile)
+    if saveFile and not saveImg.save(str(saveFile)):
+      warn('Image compositor save failed', UserWarning)
+    return pm
+
+  def show_exec(self):
+    super().show()
+    pg.mkQApp().exec_()
 
 
 class PandasTableModel(QtCore.QAbstractTableModel):
